using MasaBlazorApp3.Pojo;

namespace MasaBlazorApp3.DataAccess.Dao
{
    public interface IUserDao
    {
        int InsertUser(User user);

        Task<PageData<User>> GetAllByNickname(string nickname, int? take, int? skip);

        User GetById(int id);

        User GetByUsername(string username);

        bool UpdateUser(User user);

        bool DeleteeUser(int id);
        //重置用户密码
        bool ResetPassword(int id);

        Task<bool> UpdateSign(int id, string sign);

        bool UpdateUserPassword(User user);
        //查询当前值班信息
        HkcChangeShifts GetOnDuty();
        //保存交接班信息
        Task<bool> UpdateChangeShift(HkcChangeShifts changeShift, HkcChangeShifts changeShiftNew);
    }
}
